
using System;
using System.Collections;
using System.Threading;
using System.IO.Compression;
using System.Text.RegularExpressions;
using System.IO;
using Tamir.SharpSsh;
using System.Data.SQLite;
using Dizzy;

namespace Dizzy
{

	
	public class ProtocolException : ApplicationException
	{
		public ProtocolException () {}
		public ProtocolException (string Message) {}
	}

	public class Protocol
	{
		
		// I transfer attivi
		ArrayList transfers = new ArrayList ();
		
		public delegate void Del (string src, string dest,
		                          int transferredBytes, int totalBytes, string message);
		
		
		ArrayList threads = new ArrayList  ();
		
		Thread finder;
		
		// Dati letti dal thread che cerca per capire cosa cercare
		string keyword, user, type;
		
		GlobalConfig config;
		
		public delegate void ProtocolEvent ();
		
		// Questo evento sembra non dare problemi perché ogni volta che
		// è emesso aspettiamo che l'utente inserisca i dati necessari
		// con while(!this.config.authenticated) {}
		public event ProtocolEvent AuthenticationRequired;
		
		public Protocol (ref GlobalConfig config)
		{
			this.config = config;	
			// this.UpdateFileList ();
		}
		
		public void Search(string keyword, string user, string type)
		{
			if (this.finder != null && this.finder.IsAlive)
			{
				Log.Info ("Ricerca in corso");
				return;
			}
			if (!System.IO.File.Exists(this.config.ListFileName ()))
			{
				// Dovremmo notificare l'utente che non c'è la lista	
				Log.Warning ("La lista non è presente, chiedo di scaricarla");
			    this.UpdateFileList ();
			}
			else
			{
				Log.Info ("Avvio la ricerca nel database");
				this.keyword = keyword;
				this.user = user;
				this.type = type;
				this.finder = new Thread(new ThreadStart (this._Search));
				this.finder.Start ();
			}
			
		}
		
		protected File ReadFile(StreamReader file)
		{
			string path = file.ReadLine();
			string user = file.ReadLine ();
			int size = System.Convert.ToInt32(file.ReadLine (), 10);
			return new File(path, user, size);
		}
		
		protected void _Search()
		{
			// Questa funzione viene chiamata quando qualcuno richiede
			// una ricerca. Inoltre abbiamo la quasi certezza che ne venga
			// chiamata solo un'istanza nello stesso momento. 
			FileList list = new FileList (ref this.config);
	
			ArrayList files;
			try { files = list.Search (this.keyword, this.type, this.user); }
			catch (Exception e) {	
				// Probabilmente stiamo solamente scaricando il database, 
				// ma tant'è.
				Log.Error ("Impossibile effettuare la ricerca nel database: " + e.Message);
				files = new ArrayList ();
			}
			
			// this.fileTreeView.Clear ();
			EventManager.FileTreeViewClear ();

			foreach(File f in files)
			{
				// this.fileTreeView.AddFile (f);
				EventManager.FileTreeViewAddFile (f);
			}
			EventManager.SearchFinished ();
			if (files.Count == 0)
				EventManager.ErrorMessage ("Nessun risultato trovato!");
		}
		
		public void UpdateFileList ()
		{
			/*if(this.listUpdater != null && this.listUpdater.IsAlive)
			{
				Log.Warning ("La lista è già in fase di aggiornamento, non faccio nulla");
				return;
			}
			

			this.listUpdater = new Thread (new ThreadStart (this._UpdateFileList));
			this.listUpdater.Start (); */
			_UpdateFileList ();
		}
		
		
		
		protected void _UpdateFileList ()
		{
			// SFTPConnection s;
			Sftp s;
			
			if (!this.config.authenticated)
			{
				Log.Info ("I dati di autenticazione non sono presenti");
				this.AuthenticationRequired ();
				
				// Aspetto che l'utente abbia fatto. 
				while(!this.config.authenticated) {}
				Log.Info ("Ho acquisito utente e password");
			}
			
			EventManager.ListUpdateStarted ();
			
			// Inizializziamo la connessione con i dati che supponiamo
			// di avere. 
			// s = new SFTPConnection (this.config.GetValue("user"), 
			//                       	this.config.password);
			s = new Sftp("poisson.phc.unipi.it", this.config.GetValue("user"),
			             this.config.password);
			
			try {
				s.Connect ();
			}
			catch (Exception e) { 
				Log.Error ("Autenticazione errata (" + e.Message + ")");
				this.config.authenticated = false;
			}
			
			if(!this.config.authenticated)
			{
				EventManager.ErrorMessage ("Autenticazione fallita");
			}
			else
			{
				Log.Info ("Aggiornamento della lista avviato");
				
				// s.Download (new File("/nobackup/robol/index.db", "robol"),
				//	            this.config.ConfigDir ());
				s.Get("/nobackup/robol/index.db", this.config.ConfigDir() + 
				      System.IO.Path.DirectorySeparatorChar + "index.db");
				
				Log.Info ("Lista Aggiornata");
				s.Close ();
			}
			EventManager.ListUpdateFinished ();
		}
		

		public void Disconnect ()
		{
			foreach(FileTransfer t in this.transfers)
			{
				Log.Info ("Stopping transfers");
				t.Stop ();
			}

		}
			
		
		public void Download(File f, string downloadFolder)
		{
			// Preparo gli argomento da passare alla funzione. 
			if (!this.config.authenticated)
				this.AuthenticationRequired ();
			while(!this.config.authenticated) {}
			
			ArrayList args = new ArrayList ();
			args.Add (f);
			args.Add (downloadFolder);
			Thread t = new Thread (new ParameterizedThreadStart(_Download));
			t.Start (args);
			this.threads.Add (t);
		}
		
		protected void _Download(object args)
		{
			// Questi cast sono piuttosto brutali ma posso passare
			// solo un oggetto a Download e quindi è il meglio che
			// mi riesce di fare. 
			File f = (File) ((ArrayList) args)[0];
			string downloadFolder = (string) ((ArrayList) args)[1];
			Log.Info ("Avvio il download di " + f.name);
			
			try {
				FileDownlad transfer = new FileDownlad (f, this.config.GetValue("user"), this.config.password, downloadFolder);
				this.transfers.Add (transfer);
				transfer.Start ();
			} catch (Exception e) {		
				Log.Error (e.Message);
				this.config.authenticated = false;
				
				EventManager.ErrorMessage ("Autenticazione fallita");
			}
			
			
		}
		
		public void Upload(string f)
		{
			// Preparo gli argomento da passare alla funzione. 
			if (!this.config.authenticated)
				this.AuthenticationRequired ();
			while(!this.config.authenticated) {}
			
			ArrayList args = new ArrayList ();
			args.Add (f);
			Thread t = new Thread (new ParameterizedThreadStart(_Upload));
			t.Start (args);
			this.threads.Add (t);
		}
		
		protected void _Upload(object args)
		{
			// Questi cast sono piuttosto brutali ma posso passare
			// solo un oggetto a Download e quindi è il meglio che
			// mi riesce di fare. 
			string f = (string) ((ArrayList) args)[0];
			Log.Info ("Avvio l'upload di " + f);
			
			try {
				FileUpload transfer = new FileUpload (f, this.config.GetValue("user"), this.config.password);
				this.transfers.Add (transfer);
				transfer.Start ();
			} catch (Exception e) {
				
				Log.Error (e.Message);
				this.config.authenticated = false;
				EventManager.ErrorMessage ("Autenticazione fallita");
			}
		}
		
		
		
	}
	
	
	
	
				                
}